﻿#include <ESP8266WiFi.h>
#include <SPI.h> 

const char* ssid = "myWifi";
const char* password = "myPassword";

String sPath = "", sParam = "", sCmd = "";
boolean isNewCmd = false;
String oldPath = sPath; String oldParam = sParam;
byte volume = 150; byte saveVolume = volume;
byte channel = 0; //0=all off, 1=channel 1, 2=channel 2...
byte oldChannel = channel;
const byte ch1Pin = D1; const byte ch2Pin = D2; const byte ch3Pin = D3; const byte ch4Pin = D4; 
const int slaveSelectPin = D0;

WiFiServer server(80);

void setup() 
{
  // SPI
  pinMode(slaveSelectPin, OUTPUT);
  SPI.begin(); 

  // Serial
  Serial.begin(115200);
  delay(10);

  //Pins for relays
  pinMode(ch1Pin, OUTPUT); pinMode(ch2Pin, OUTPUT); pinMode(ch3Pin, OUTPUT); pinMode(ch4Pin, OUTPUT);
  digitalWrite(ch1Pin, LOW); digitalWrite(ch2Pin, LOW); digitalWrite(ch3Pin, LOW); digitalWrite(ch4Pin, LOW);

  // inital connect for WIFI
  WiFi.mode(WIFI_STA);
  WiFiStart();
}

void loop() 
{
  // if not connected --> return and start again
  if (WiFi.status() != WL_CONNECTED)
  {
    WiFiStart();
  }

  // if not client connected --> return and again
  WiFiClient client = server.available();
  if (!client) 
  {
    return;
  }
  //check timeout --> for client-connection timeout
  unsigned long ultimeout = millis()+250;
  while(!client.available() && (millis()<ultimeout) ) 
  {
    delay(1);
  }
  if(millis()>ultimeout) 
  {
    return; 
  }
  String sRequest = client.readStringUntil('\r');
  client.flush();
  if(sRequest=="")
  {
    client.stop();
    return;
  }
  // get path; end of path is either space or ?
  // Syntax is e.g. GET /?pin=MOTOR1STOP HTTP/1.1
  isNewCmd = false;
  String sGetstart="GET /?"; //Anfang des eigentlichen GET-Strings mit Inhalt
  int iStart, iEndSpace, iEndQuest;
  iStart = sRequest.indexOf(sGetstart); //Position 1. Zeichen des Strings, meist von GET --> i.d.R. Null (0)
  if (iStart>=0) 
  {    //GET gefunden
    iStart+=+sGetstart.length();  //erhöhen auf Zeichen hinter GET /? 
    iEndSpace = sRequest.indexOf(" ",iStart); //Ende des zu verwertenden Strings
    iEndQuest = sRequest.indexOf("=",iStart); //= als Trenner zwischen Parameter und Cmd
    sPath  = sRequest.substring(iStart,iEndQuest);
    sParam = sRequest.substring(iEndQuest + 1,iEndSpace);
    if (sPath == "volume"){
      if (sParam == "plus") {
        if (volume < 255) {
          volume++;
          saveVolume = volume;
        }
      }
      if (sParam == "minus") {
        if (volume > 0) {
          volume--;
          saveVolume = volume;
        }
      }
      if (sParam == "PLUS") {
        if (volume < 251) {
          volume = volume + 5;
          saveVolume = volume;
        }
      }
      if (sParam == "MINUS") {
        if (volume > 5) {
          volume = volume - 5;
          saveVolume = volume;
        }
      }
    }
    if (sPath == "mute") {
      if (sParam.toInt() == 1) {
        volume = saveVolume;
      }
      if (sParam.toInt() == 0) {
        volume = 0;
      }
    }
    if (sPath == "channel") {
      if ((sParam.toInt() <= 4) && (sParam.toInt() >= 0)) {
        channel = sParam.toInt();
      }
      if (channel != oldChannel) {
        oldChannel = channel;
        digitalWrite(ch1Pin, LOW); digitalWrite(ch2Pin, LOW); digitalWrite(ch3Pin, LOW); digitalWrite(ch4Pin, LOW); 
        delay(50);   //debounce
        switch (channel) {
          case 1:
            digitalWrite(ch1Pin, HIGH);
            break;
          case 2:
            digitalWrite(ch2Pin, HIGH);
            break;
           case 3:
            digitalWrite(ch3Pin, HIGH);
            break;
          case 4:
            digitalWrite(ch4Pin, HIGH);
            break;
       }
      }
    }
    delay(50);
    pga2310Write(volume);
  } //Get found

  // format the html response
  String sResponse, sHeader;
  
  // 404 for non-matching path
  if(!((sPath == "volume") || (sPath == "mute") || (sPath == "channel") || (sPath == "")))
  {
    //Serial.println("404 Not found");
    sResponse="<html><head><title>404 Not Found</title></head><body><h1>404 Not Found</h1><p>The requested URL was not found on this server.</p></body></html>";
    sHeader  = "HTTP/1.1 404 Not found\r\n";
    sHeader += "Content-Length: ";
    sHeader += sResponse.length();
    sHeader += "\r\n";
    sHeader += "Content-Type: text/html\r\n";
    sHeader += "Connection: close\r\n";
    sHeader += "\r\n";
  }

  // format the html page
  else
  {
    sResponse  = "<html><head><title>Controlling PGA2310 with ESP8266</title></head><body>";
    sResponse += "<font color=\"#000000\"><body bgcolor=\"#CCCCCC\">";
    sResponse += "<meta name=\"viewport\" content=\"width=device-width, initial-scale=1.0, user-scalable=yes\">";
    sResponse += "<h3>Controlling PGA2310 with ESP8266</h3>";
    sResponse += "Command: " + sPath + "<BR>";
    sResponse += "Value:     " + sParam + "<BR>";
    sResponse += "<BR>";
    sResponse += "<FONT SIZE=+1>";
    sResponse += "Volume: " + String(volume) + " (0 ... 255)<BR>";
    sResponse += "<FONT SIZE=-1>";
    sResponse += "<p>Control:<BR><BR><a href=\"?volume=plus\"><button>+1</button></a>&nbsp;<a href=\"?volume=minus\"><button>-1</button></a></p>";
    sResponse += "<p><a href=\"?volume=PLUS\"><button>+5</button></a>&nbsp;<a href=\"?volume=MINUS\"><button>-5</button></a></p>";
    sResponse += "<p><a href=\"?mute=1\"><button>ON</button></a>&nbsp;<a href=\"?mute=0\"><button>OFF</button></a></p>";    
    sResponse += "<BR><BR>";
    sResponse += "<p>Channel: " + String(channel) + "<BR><BR><a href=\"?channel=0\"><button>All off</button></a>&nbsp;<a href=\"?channel=1\"><button>Channel 1</button></a>"; 
    sResponse += "<a href=\"?channel=2\"><button>Channel 2</button></a>&nbsp;<a href=\"?channel=3\"><button>Channel 3</button></a>&nbsp;<a href=\"?channel=4\"><button>Channel 4</button></a></p>"; 
    sResponse += "</body></html>";
    
    sHeader  = "HTTP/1.1 200 OK\r\n";
    sHeader += "Content-Length: ";
    sHeader += sResponse.length();
    sHeader += "\r\n";
    sHeader += "Content-Type: text/html\r\n";
    sHeader += "Connection: close\r\n";
    sHeader += "\r\n";
  }
  
  // Send the response to the client
  client.print(sHeader);
  client.print(sResponse);
  
  // and stop the client
  client.stop();
}

void pga2310Write(int value) {
  digitalWrite(slaveSelectPin, LOW);
  SPI.transfer(value);
  SPI.transfer(value);
  digitalWrite(slaveSelectPin, HIGH);
}

void WiFiStart()
{
  Serial.println();
  Serial.print("Connecting to ");
  Serial.println(ssid);
  
  WiFi.begin(ssid, password);
  
  while (WiFi.status() != WL_CONNECTED) {
    delay(500);
    Serial.print(".");
  }
  Serial.println("");
  Serial.println("WiFi connected");
  
  server.begin();
  Serial.println("Server started");
  Serial.println(WiFi.localIP());
}


